// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef HU_VARADIISTVAN_SCPL_IO_WIN32_OVERLAPPABLE_H
#define HU_VARADIISTVAN_SCPL_IO_WIN32_OVERLAPPABLE_H
//------------------------------------------------------------------------------

#include "EventFailable.h"

#include <windows.h>

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

//------------------------------------------------------------------------------

class Waiter;

//------------------------------------------------------------------------------

/**
 * Base class for classes that maintain a handle and may be subject to
 * overlapped I/O operations whose completion is waited for in a
 * Waiter.
 */
class Overlappable : public EventFailable
{
protected:
    /**
     * The waiter to be used for waiting for events.
     */
    Waiter* waiter;

    /**
     * The handle to use.
     */
    HANDLE handle;

protected:
    /**
     * Construct the object.
     */
    Overlappable(std::unique_ptr<EventFactory> eventFactory,
                 Waiter* waiter = 0, HANDLE handle = 0);

public:
    /**
     * Destroy the object.
     */
    virtual ~Overlappable();

    /**
     * Get the waiter.
     */
    Waiter* getWaiter();

    /**
     * Determine if the overlappable has a waiter.
     */
    bool hasWaiter() const;

    /**
     * Get the handle and clear it.
     */
    HANDLE releaseHandle();

    friend class Completer;
};

//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline Overlappable::Overlappable(std::unique_ptr<EventFactory> eventFactory,
                                  Waiter* waiter, HANDLE handle) :
    EventFailable(std::move(eventFactory)),
    waiter(waiter),
    handle(handle)
{
}

//------------------------------------------------------------------------------

inline Overlappable::~Overlappable()
{
    if (handle!=0) CloseHandle(handle);
}

//------------------------------------------------------------------------------

inline Waiter* Overlappable::getWaiter()
{
    return waiter;
}

//------------------------------------------------------------------------------

inline bool Overlappable::hasWaiter() const
{
    return waiter != 0;
}

//------------------------------------------------------------------------------

inline HANDLE Overlappable::releaseHandle()
{
    HANDLE h = handle;
    handle = 0;
    return h;
}

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------
#endif // HU_VARADIISTVAN_SCPL_IO_WIN32_OVERLAPPABLE_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
