// Copyright (c) 2022 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#pragma once

//------------------------------------------------------------------------------

#include "../BufferedStreamAcceptor.h"

#include "Completer.h"

#include "TCPServerSocketBase.h"
#include "TCPSocket.h"

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

//------------------------------------------------------------------------------

class TCPServerSocket;

//------------------------------------------------------------------------------

/**
 * An acceptor that provides a TCPSocket for each accepted connection
 */
class TCPAcceptor : public Completer,
                    public BufferedStreamAcceptor
{
private:
    /**
     * The accepted socket.
     */
    HANDLE acceptedSocket;

private:
    /**
     * Construct the acceptor.
     */
    TCPAcceptor(TCPServerSocketBase* socket);

    /**
     * Get the socket.
     *
     * It returns the overlappable from the Completer part, knowing
     * that it must be a TCPServerSocket.
     */
    TCPServerSocketBase& getServerSocket() const;

public:
    /**
     * Try to accept a connection.
     *
     * If one is already accepted, but not retrieved yet, return true.
     *
     * If a new connection becomes immediately available, return true.
     *
     * If a new connection is not yet available, returns false.
     *
     * If some error occurs, the acceptor is marked failed and false
     * is returned.
     */
    bool accept();

    /**
     * Get the TCP socket accepted last. If no socket was accepted,
     * return 0. The TCP socket's waiter will be the given one
     * (which can be 0).
     */
    TCPSocket* getSocket(Waiter* waiter,
                         size_t readingCapacity = TCPSocket::DEFAULT_CAPACITY,
                         size_t writingCapacity = TCPSocket::DEFAULT_CAPACITY);
    /**
     * Get the TCP socket accepted last. If no socket was accepted,
     * return 0. The TCP socket's waiter will be the same as that of
     * the associated server socket.
     */
    TCPSocket* getSocket(size_t readingCapacity = TCPSocket::DEFAULT_CAPACITY,
                         size_t writingCapacity = TCPSocket::DEFAULT_CAPACITY);

    /**
     * Get the last accepted socket as a buffered stream with the given waiter,
     * if any.
     */
    BufferedStream* getAcceptedBufferedStream(Waiter* waiter = nullptr) override;

protected:
    /**
     * Set the indicator of an accepted connection to true.
     *
     * @see Completer::handleWaitedResult
     */
    void handleWaitedResult(DWORD size) override;

    friend class TCPServerSocket;
};


//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline TCPAcceptor::TCPAcceptor(TCPServerSocketBase* socket) :
    Completer(socket),
    acceptedSocket(WSAInterface::get().getInvalidSocket())
{
}

//------------------------------------------------------------------------------

inline TCPServerSocketBase& TCPAcceptor::getServerSocket() const
{
    return static_cast<TCPServerSocketBase&>(overlappable);
}

//------------------------------------------------------------------------------

inline TCPSocket* TCPAcceptor::getSocket(Waiter* waiter,
                                         size_t readingCapacity,
                                         size_t writingCapacity)
{
    if (acceptedSocket==WSAInterface::get().getInvalidSocket()) return 0;

    TCPSocket* socket = new TCPSocket(acceptedSocket, waiter,
                                      readingCapacity, writingCapacity);
    acceptedSocket = WSAInterface::get().getInvalidSocket();
    return socket;
}

//------------------------------------------------------------------------------

inline TCPSocket* TCPAcceptor::getSocket(size_t readingCapacity,
                                         size_t writingCapacity)
{
    return getSocket(overlappable.getWaiter(), readingCapacity, writingCapacity);
}

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
